pre .hll { background-color: #ffffcc }
pre .c { color: #408080; font-style: italic } /* Comment */
pre .err { border: 1px solid #FF0000 } /* Error */
pre .k { color: #008000; font-weight: bold } /* Keyword */
pre .o { color: #666666 } /* Operator */
pre .cm { color: #408080; font-style: italic } /* Comment.Multiline */
pre .cp { color: #BC7A00 } /* Comment.Preproc */
pre .c1 { color: #408080; font-style: italic } /* Comment.Single */
pre .cs { color: #408080; font-style: italic } /* Comment.Special */
pre .gd { color: #A00000 } /* Generic.Deleted */
pre .ge { font-style: italic } /* Generic.Emph */
pre .gr { color: #FF0000 } /* Generic.Error */
pre .gh { color: #000080; font-weight: bold } /* Generic.Heading */
pre .gi { color: #00A000 } /* Generic.Inserted */
pre .go { color: #888888 } /* Generic.Output */
pre .gp { color: #000080; font-weight: bold } /* Generic.Prompt */
pre .gs { font-weight: bold } /* Generic.Strong */
pre .gu { color: #800080; font-weight: bold } /* Generic.Subheading */
pre .gt { color: #0044DD } /* Generic.Traceback */
pre .kc { color: #008000; font-weight: bold } /* Keyword.Constant */
pre .kd { color: #008000; font-weight: bold } /* Keyword.Declaration */
pre .kn { color: #008000; font-weight: bold } /* Keyword.Namespace */
pre .kp { color: #008000 } /* Keyword.Pseudo */
pre .kr { color: #008000; font-weight: bold } /* Keyword.Reserved */
pre .kt { color: #B00040 } /* Keyword.Type */
pre .m { color: #666666 } /* Literal.Number */
pre .s { color: #BA2121 } /* Literal.String */
pre .na { color: #7D9029 } /* Name.Attribute */
pre .nb { color: #008000 } /* Name.Builtin */
pre .nc { color: #0000FF; font-weight: bold } /* Name.Class */
pre .no { color: #880000 } /* Name.Constant */
pre .nd { color: #AA22FF } /* Name.Decorator */
pre .ni { color: #999999; font-weight: bold } /* Name.Entity */
pre .ne { color: #D2413A; font-weight: bold } /* Name.Exception */
pre .nf { color: #0000FF } /* Name.Function */
pre .nl { color: #A0A000 } /* Name.Label */
pre .nn { color: #0000FF; font-weight: bold } /* Name.Namespace */
pre .nt { color: #008000; font-weight: bold } /* Name.Tag */
pre .nv { color: #19177C } /* Name.Variable */
pre .ow { color: #AA22FF; font-weight: bold } /* Operator.Word */
pre .w { color: #bbbbbb } /* Text.Whitespace */
pre .mf { color: #666666 } /* Literal.Number.Float */
pre .mh { color: #666666 } /* Literal.Number.Hex */
pre .mi { color: #666666 } /* Literal.Number.Integer */
pre .mo { color: #666666 } /* Literal.Number.Oct */
pre .sb { color: #BA2121 } /* Literal.String.Backtick */
pre .sc { color: #BA2121 } /* Literal.String.Char */
pre .sd { color: #BA2121; font-style: italic } /* Literal.String.Doc */
pre .s2 { color: #BA2121 } /* Literal.String.Double */
pre .se { color: #BB6622; font-weight: bold } /* Literal.String.Escape */
pre .sh { color: #BA2121 } /* Literal.String.Heredoc */
pre .si { color: #BB6688; font-weight: bold } /* Literal.String.Interpol */
pre .sx { color: #008000 } /* Literal.String.Other */
pre .sr { color: #BB6688 } /* Literal.String.Regex */
pre .s1 { color: #BA2121 } /* Literal.String.Single */
pre .ss { color: #19177C } /* Literal.String.Symbol */
pre .bp { color: #008000 } /* Name.Builtin.Pseudo */
pre .vc { color: #19177C } /* Name.Variable.Class */
pre .vg { color: #19177C } /* Name.Variable.Global */
pre .vi { color: #19177C } /* Name.Variable.Instance */
pre .il { color: #666666 } /* Literal.Number.Integer.Long */
