"""This module just has a big list of the intrinsic functions to save space in sourceform.py
"""

INTRINSICS = [
    "abort",
    "abs",
    "abstract",
    "access",
    "achar",
    "acos",
    "acosh",
    "adjustl",
    "adjustr",
    "aimag",
    "aint",
    "alarm",
    "all",
    "allocatable",
    "allocate",
    "allocated",
    "and",
    "anint",
    "any",
    "asin",
    "asinh",
    "assign",
    "associate",
    "associated",
    "asynchronous",
    "atan",
    "atan2",
    "atanh",
    "atomic_add",
    "atomic_and",
    "atomic_cas",
    "atomic_define",
    "atomic_fetch_add",
    "atomic_fetch_and",
    "atomic_fetch_or",
    "atomic_fetch_xor",
    "atomic_or",
    "atomic_ref",
    "atomic_xor",
    "backtrace",
    "backspace",
    "bessel_j0",
    "bessel_j1",
    "bessel_jn",
    "bessel_y0",
    "bessel_y1",
    "bessel_yn",
    "bge",
    "bgt",
    "bind",
    "bit_size",
    "ble",
    "block",
    "block data",
    "blt",
    "btest",
    "c_associated",
    "c_f_pointer",
    "c_f_procpointer",
    "c_funloc",
    "c_loc",
    "c_sizeof",
    "cabs",
    "call",
    "case",
    "case default",
    "cdabs",
    "ceiling",
    "char",
    "character",
    "chdir",
    "chmod",
    "class",
    "close",
    "cmplx",
    "codimension",
    "co_broadcast",
    "co_max",
    "co_min",
    "co_reduce",
    "co_sum",
    "command_argument_count",
    "common",
    "compiler_options",
    "compiler_version",
    "complex",
    "concurrent",
    "conjg",
    "contains",
    "contiguous",
    "continue",
    "cos",
    "cosh",
    "count",
    "cpu_time",
    "critical",
    "cshift",
    "cycle",
    "data",
    "ctime",
    "dabs",
    "date_and_time",
    "dble",
    "dcmplx",
    "deallocate",
    "deferred",
    "digits",
    "dim",
    "dimension",
    "do",
    "dlog",
    "dlog10",
    "dmax1",
    "dmin1",
    "dot_product",
    "double complex",
    "double precision",
    "dprod",
    "dreal",
    "dshiftl",
    "dshiftr",
    "dsqrt",
    "dtime",
    "elemental",
    "else",
    "else if",
    "elseif",
    "elsewhere",
    "end",
    "end associate",
    "end block",
    "end block data",
    "end critical",
    "end do",
    "end enum",
    "end forall",
    "end function",
    "end if",
    "end interface",
    "end module",
    "end program",
    "end select",
    "end submodule",
    "end subroutine",
    "end type",
    "end where",
    "endfile",
    "endif",
    "entry",
    "enum",
    "enumerator",
    "eoshift",
    "epsilon",
    "equivalence",
    "erf",
    "erfc",
    "erfc_scaled",
    "etime",
    "error stop",
    "execute_command_line",
    "exit",
    "exp",
    "exponent",
    "extends",
    "extends_type_of",
    "external",
    "fget",
    "fgetc",
    "final",
    "findloc",
    "fdate",
    "floor",
    "flush",
    "fnum",
    "forall",
    "format",
    "fput",
    "fputc",
    "fraction",
    "function",
    "free",
    "fseek",
    "fstat",
    "ftell",
    "gamma",
    "generic",
    "gerror",
    "getarg",
    "get_command",
    "get_command_argument",
    "getcwd",
    "getenv",
    "get_environment_variable",
    "go to",
    "goto",
    "getgid",
    "getlog",
    "getpid",
    "getuid",
    "gmtime",
    "hostnm",
    "huge",
    "hypot",
    "iabs",
    "iachar",
    "iall",
    "iand",
    "iany",
    "iargc",
    "ibclr",
    "ibits",
    "ibset",
    "ichar",
    "idate",
    "ieee_class",
    "ieee_copy_sign",
    "ieee_get_flag",
    "ieee_get_halting_mode",
    "ieee_get_rounding_mode",
    "ieee_get_status",
    "ieee_get_underflow_mode",
    "ieee_is_finite",
    "ieee_is_nan",
    "ieee_is_negative",
    "ieee_is_normal",
    "ieee_logb",
    "ieee_next_after",
    "ieee_rem",
    "ieee_rint",
    "ieee_scalb",
    "ieee_selected_real_kind",
    "ieee_set_flag",
    "ieee_set_halting_mode",
    "ieee_set_rounding_mode",
    "ieee_set_status",
    "ieee_support_datatype",
    "ieee_support_denormal",
    "ieee_support_divide",
    "ieee_support_flag",
    "ieee_support_halting",
    "ieee_support_inf",
    "ieee_support_io",
    "ieee_support_nan",
    "ieee_support_rounding",
    "ieee_support_sqrt",
    "ieee_support_standard",
    "ieee_support_underflow_control",
    "ieee_unordered",
    "ieee_value",
    "ieor",
    "ierrno",
    "if",
    "imag",
    "image_index",
    "implicit",
    "implicit none",
    "import",
    "include",
    "index",
    "inquire",
    "int",
    "integer",
    "intent",
    "interface",
    "intrinsic",
    "int2",
    "int8",
    "ior",
    "iparity",
    "irand",
    "is",
    "is_contiguous",
    "is_iostat_end",
    "is_iostat_eor",
    "isatty",
    "ishft",
    "ishftc",
    "isnan",
    "itime",
    "kill",
    "kind",
    "lbound",
    "lcobound",
    "leadz",
    "len",
    "len_trim",
    "lge",
    "lgt",
    "link",
    "lle",
    "llt",
    "lock",
    "lnblnk",
    "loc",
    "log",
    "log_gamma",
    "log10",
    "logical",
    "long",
    "lshift",
    "lstat",
    "ltime",
    "malloc",
    "maskl",
    "maskr",
    "matmul",
    "max",
    "max0",
    "maxexponent",
    "maxloc",
    "maxval",
    "mclock",
    "mclock8",
    "merge",
    "merge_bits",
    "min",
    "min0",
    "minexponent",
    "minloc",
    "minval",
    "mod",
    "module",
    "module procedure",
    "modulo",
    "move_alloc",
    "mvbits",
    "namelist",
    "nearest",
    "new_line",
    "nint",
    "non_overridable",
    "none",
    "nopass",
    "norm2",
    "not",
    "null",
    "nullify",
    "num_images",
    "only",
    "open",
    "or",
    "operator",
    "optional",
    "pack",
    "parameter",
    "parity",
    "pass",
    "pause",
    "pointer",
    "perror",
    "popcnt",
    "poppar",
    "precision",
    "present",
    "print",
    "private",
    "procedure",
    "product",
    "program",
    "protected",
    "public",
    "pure",
    "radix",
    "ran",
    "rand",
    "random_number",
    "random_seed",
    "range",
    "rank",
    "read",
    "real",
    "recursive",
    "rename",
    "repeat",
    "reshape",
    "result",
    "return",
    "rewind",
    "rewrite",
    "rrspacing",
    "rshift",
    "same_type_as",
    "save",
    "scale",
    "scan",
    "secnds",
    "second",
    "select",
    "select case",
    "select type",
    "selected_char_kind",
    "selected_int_kind",
    "selected_real_kind",
    "sequence",
    "set_exponent",
    "shape",
    "shifta",
    "shiftl",
    "shiftr",
    "sign",
    "signal",
    "sin",
    "sinh",
    "size",
    "sizeof",
    "sleep",
    "spacing",
    "spread",
    "sqrt",
    "srand",
    "stat",
    "stop",
    "storage_size",
    "submodule",
    "subroutine",
    "sum",
    "sync all",
    "sync images",
    "sync memory",
    "symlnk",
    "system",
    "system_clock",
    "tan",
    "tanh",
    "target",
    "then",
    "this_image",
    "time",
    "time8",
    "tiny",
    "trailz",
    "transfer",
    "transpose",
    "trim",
    "ttynam",
    "type",
    "type_as",
    "ubound",
    "ucobound",
    "umask",
    "unlock",
    "unlink",
    "unpack",
    "use",
    "value",
    "verify",
    "volatile",
    "wait",
    "where",
    "while",
    "write",
    "xor",
    "zabs",
]
