{% extends "base.html" %}
{% block title %}
All Modules &ndash; {{ project }}
{% endblock %}
{% block body %}
{% macro mod_entry(mod,level) %}
    <tr><td>{{ mod }}</td><td>{{ mod.parent }}</td><td>{{ mod.meta['summary'] }}</td></tr>
    {% for m in mod.descendants %}
        
    {% endfor %}
{% endmacro %}
      <div class="row">
        <div class="col-lg-12" id='text'>
			 <h1>Modules</h1>
			 <table class="table">
			 <thead><tr><th>Module</th><th>Source File</th><th>Description</th></tr></thead>
			 <tbody>
             {% set row_class = cycler('active', '') %}
			 {% for mod in project.modules|sort(attribute='name') recursive %}
			   <tr class="{% if loop.depth == 1 %}{{ row_class.current }}{% else %}{{ row_class.current }} submod{% endif %}"><td>{% for i in range(loop.depth0) -%}&nbsp;&nbsp;&nbsp;{%- endfor %}{{ mod }}</td><td>{{ mod.parent }}</td><td>{{ mod.meta['summary'] }}</td></tr>
               {% if mod.descendants %}
                  {{ loop(mod.descendants) }}
               {% endif %}
               {%if loop.depth == 1 %}<!--{{row_class.next()}}-->{% endif %}
			 {% endfor %}
			 </tbody></table>
             {{ project.usegraph }}
        </div>
      </div>
{% endblock %}
